

#include <stdio.h>  // for printf()
#include <stdlib.h> // for exit()


#include "CAENVMElib.h"


void Usage(char *progname) {

    fprintf(stderr,"Usage -l Link -b BdNum -a VMEBaseAddress\n%s  \n",progname);
    exit(1);
}

int main(int argc, char **argv) {
   int rc;
   CVBoardTypes V1718 = cvV1718;
   CVBoardTypes V2718 = cvV2718;
   short Link = 0;
   short BdNum = 0;
   int32_t VMEHandle = 0;

   char SwRel[8];
   char FwRel[8];

   unsigned long VMEBaseAddress = 0x0;
   unsigned long VMEAddress;
   CVAddressModifier AddrMod =  cvA32_U_DATA;
   CVDataWidth DataWidth= cvD32;

   unsigned int Data32;
   unsigned short Data16;

   int i;

   if (argc > 1) {
        for(i=1;i <(argc-1);i++) {
            if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
                switch(tolower(argv[i][1])) {
                    case 'l':
                        Link = atoi(argv[++i]);
                        break;
                    case 'b':
                        BdNum = atoi(argv[++i]);
                        break;
                    case 'a':
                        VMEBaseAddress = strtol(argv[++i], (char **)NULL, 16);
                        break;
                    default:
                        Usage(argv[0]);
                        break;
                }
            }
            else
                Usage(argv[0]);
        }
   }
   
   rc = CAENVME_SWRelease(SwRel);
   printf("CAENVME_SwRelease returns code %d\n", rc);
   printf("CAENVME software release %s\n", &SwRel[0]);

   printf("Looking for CAEN V2718 bridge\n");

   rc = CAENVME_Init(V2718, Link, BdNum, &VMEHandle);
   if (rc != 0) {
       printf("CAENVME_Init returns code %d for V2718\n", rc);
       printf("CAEN V2718 not found so looking for CAEN V1718 bridge\n");
       rc = CAENVME_Init(V1718, Link, BdNum, &VMEHandle);
       if (rc != 0) {
           printf("CAENVME_Init returns code %d for V1718\n", rc);
           printf("CAEN V1718 not found - NO useful hardware found\n");
           exit(0);
       } else {
           printf("CAENVME_Init connected to V1718\n");
       }
   } else {
       printf("CAENVME_Init connected to V2718\n");
   }
   
   rc = CAENVME_BoardFWRelease(VMEHandle, FwRel);
   printf("CAENVME_BoardFWRelease returns code %d\n", rc);
   printf("CAENVME firmware release %s\n", &FwRel[0]);

   printf("Accessing V1724 module at VME base address 0x%08x\n", VMEBaseAddress);

   VMEAddress = VMEBaseAddress + 0x108c;

   rc  = CAENVME_ReadCycle(VMEHandle, VMEAddress, &Data32, AddrMod, DataWidth);
   if (rc == 0) {
      printf("CAENVME_ReadCycle accessing address 0x%08x returns 0x%08x\n", VMEAddress, Data32);
   } else {
      printf("CAENVME_ReadCycle accessing address 0x%08x returns code %d\n", VMEAddress, rc);
   }

   VMEAddress = VMEBaseAddress + 0x8124;

   rc  = CAENVME_ReadCycle(VMEHandle, VMEAddress, &Data32, AddrMod, DataWidth);
   if (rc == 0) {
      printf("CAENVME_ReadCycle accessing address 0x%08x returns 0x%08x\n", VMEAddress, Data32);
   } else {
      printf("CAENVME_ReadCycle accessing address 0x%08x returns code %d\n", VMEAddress, rc);
   }

   VMEAddress = 0x02001000;

   rc  = CAENVME_ReadCycle(VMEHandle, VMEAddress, &Data16, cvA32_U_DATA, cvD16);
   if (rc == 0) {
      printf("CAENVME_ReadCycle accessing address 0x%08x returns 0x%04x\n", VMEAddress, Data16);
   } else {
      printf("CAENVME_ReadCycle accessing address 0x%08x returns code %d\n", VMEAddress, rc);
   }
            
   rc  = CAENVME_End(VMEHandle);
   printf("CAENVME_End returns code %d\n", rc);

   printf("Probe completed OK\n");

   exit(0);
}
