
proc Set_V785_Options {} {
    global Options V785_Base

    load_Options

   if {![info exists Options(v785.channels)]} {set Options(v785.channels) 4096}
   if {![info exists Options(v785.offset)]} {set Options(v785.offset) 0}
   if {![info exists Options(v785.shift)]} {set Options(v785.shift) 0}
   if {![info exists Options(v785_GroupBase)]} {set Options(v785_GroupBase) $V785_Base}
   set V785_Base   $Options(v785_GroupBase)

    update_Options
}


proc V785_DAQ-go {} {
    global VMEserver

    for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/v785#$i.Status1'  ?object)" } m]
      if {$z!=0} then {break}

      write-registers v785#$i.Bit_Set_2 4     ;# clear the data
      write-registers v785#$i.Bit_Clear_2 4

      write-registers v785#$i.Bit_Set_2 0x1000   ;#  always generate event header
      write-registers v785#$i.Ctrl1 0x24     ;#  enable BLKEND and BERR 
    }
}

proc V785_DAQ-halt {} {
    global VMEserver

    for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/v785#$i.Status1'  ?object)" } m]
      if {$z!=0} then {break}

    }
}

proc V785_DAQ-setup {} {
    global VMEserver

    for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/v785#$i.Status1'  ?object)" } m]
      if {$z!=0} then {break}

      write-registers v785#$i.Bit_Set_1 0x80     ;# software reset the board
      write-registers v785#$i.Bit_Clear_1 0x80

      write-registers v785#$i.Bit_Set_2 0x1000   ;#  always generate event header
      write-registers v785#$i.Bit_Set_2 0x8      ;# no overflow suppression 
    }
}

proc V785_DAQ-flush {} {
    global VMEserver

    for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/v785#$i.Status1'  ?object)" } m]
      if {$z!=0} then {break}

      write-registers v785#$i.Bit_Set_2 4     ;# clear the data
      write-registers v785#$i.Bit_Clear_2 4
    }
}

proc V785_DAQ-reset {} {}

proc Dump_Raw_VME_Data_V785 {data} {
     
     set info ""

     foreach word $data {
         if {[string range $word 0 1] != "0x"} then {set word 0x$word}
         if {$word == 0xaa5555aa} then {
             append info "$word - end of dump\n"
             break
         }
         if {$word == 0x55aaaa55} then {
             append info "$word - end of event\n"
             continue
         }
         set type [expr $word >> 24 & 7]
         switch $type {
            0 {
             set event_type "data"
             set channel [expr $word >> 16 & 0x3f]
             set un [expr $word >> 13 & 1]
             set ov [expr $word >> 12 & 1]
             set adc [expr $word  & 0xfff]
             append info "[format "0x%8x    %8s   channel= %5i; un= %3i; ov= %3i; data= 0x%5x" $word $event_type $channel $un $ov $adc]"
            }
            2 {
             set event_type "header"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word >> 8 & 0x3f]
             append info "[format "0x%8x    %8s   module= %5i; channels= %6i" $word $event_type $geo $counter]"
            }
            4 {
             set event_type "trailer"
             set geo [expr $word >> 27 & 0x1f]
             set counter [expr $word & 0xffffff]
             append info "[format "0x%8x    %8s   module= %5i; event data counter= %10i" $word $event_type $geo $counter]"
            }
            default {
             append info "$word - not valid datum\n"
            }
         }
         append info "\n"
     }
   return $info
}


proc define_V785_config {} {
   global VMEserver
    global V785_Base

#   set maxmodule [inform vmecontrol inquire v785count]
#   if {[string length $maxmodule] > 2} {set maxmodule 0}

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/v785#$i.Status1'  ?object)" } m]
      if {$z!=0} then {break}
      incr maxmodule
   }

   if {$maxmodule > 0} then {DefineConfig "785 0 [format "0x%x" $V785_Base]"}

   for {set i 1} {$i <= $maxmodule} {incr i} {

      set z [catch {eg "(register :name '/$VMEserver/v785#$i.Output_Buffer'  ?object)" } m]
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to locate register v785#$i.Output_Buffer"
          clear-footer
          error $m
      }
      DefineConfig "785 $i [string range [first $m] 2 end]"
   }
}


proc create_V785_spectra {} {
    global SpectrumServer VMEserver V785_Base
    global Options 


#   set maxmodule [inform vmecontrol inquire v785count]
#   if {[string length $maxmodule] > 2} {set maxmodule 0}

   set maxmodule 0
   for {set i 1} {$i <= 16} {incr i} {
      set z [catch {eg "(register :name '/$VMEserver/v785#$i.Status1'  ?object)" } m]
      if {$z!=0} then {break}
      incr maxmodule
   }
   set maxadc 32

   set UseGroup [read-register NetVar.UseGroup]

   for {set i 1} {$i <= $maxmodule} {incr i} {

    set-footer "creating spectra for module v785#$i"

    for {set j 0} {$j < $maxadc} {incr j} {

      set spectrumname [SpectrumName v785#$i.adc$j]
      set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $Options(v785.channels) :name '$spectrumname' !new)" } m]

#    Allow "Spectrum already exists" so that further spectra can be created
      if {$z!=0 && $m==0x30004} then {set z 0}
      if {$z!=0} then {
          display-eg-error-report self $m "Attempting to create spectrum  v785#$i.adc$j"
          clear-footer
          error $m
      } else {
          if {$UseGroup == 0} {
              set group [expr $V785_Base - 1 + $i]
              map_spectrum  "$spectrumname $group $j $Options(v785.offset) $Options(v785.shift)"
          } else {
              set group [expr (($i - 1) * 32) + $V785_Base + $j]
              map_spectrum  "$spectrumname $group 0 $Options(v785.offset) $Options(v785.shift)"
          }
      }
    }
   }
}
