
#    This is a component of Spectrum.tcl

if {$Started == 0} {
    
    variable backpeakoptions
    variable peakfindoptions
    variable brumfitoptions

      set backpeakoptions(backtype) flatline
      set backpeakoptions(peaktype) gauss
      set backpeakoptions(weight)   count
      set backpeakoptions(shape)    differen
      set backpeakoptions(peaks)    1
      set backpeakoptions(order)    1
      set backpeakoptions(shapesfile) "no"

      set peakfindoptions(low-chan) 0
      set peakfindoptions(high-chan) 8191
      set peakfindoptions(fwhm)     5
      set peakfindoptions(accept)   10
      
      set brumfitoptions(shape) gauss
      set brumfitoptions(widthopt) different
      set brumfitoptions(peakopt) different
      set brumfitoptions(order) 2
      set brumfitoptions(widths) 5.0
      set brumfitoptions(skews) 10.0

}
    
    proc SpectrumReference {} {
       variable currentgallery
       variable currentoverlap
        upvar #1 dbSpecPathName($currentgallery,$currentoverlap) SpecPathName
        return $SpecPathName
    }

   proc do_view {args} {
       variable currentgallery
       variable currentoverlap
       
       upvar #1 dbXMin($currentgallery) XMin
       upvar #1 dbXMax($currentgallery) XMax
       upvar #1 dbYMin($currentgallery) YMin
       upvar #1 dbYMax($currentgallery) YMax

       variable SpecLimit
       variable WindowSize
       variable CustomYView
       variable QuikLoad

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           "expand"        {do_expand}
            zoomin -
           "zoom in"       {do_zoom In}
            zoomout -
           "zoom out"      {do_zoom Out}
            times2         {do_YMax [expr $YMax*2]; set CustomYView 1}
            divide2        {do_YMax [expr $YMax/2]; set CustomYView 1}
            left           {do_XMin [expr $XMin - ($WindowSize/4)]; do_XMax [expr $XMax - ($WindowSize/4)]}
            right          {do_XMin [expr $XMin + ($WindowSize/4)]; do_XMax [expr $XMax + ($WindowSize/4)]}
            up             {do_YMin [expr $YMin - ($SpecLimit/4)]; do_YMax [expr $YMax - ($SpecLimit/4)]; set CustomYView 1}
            down           {do_YMin [expr $YMin + ($SpecLimit/4)]; do_YMax [expr $YMax + ($SpecLimit/4)]; set CustomYView 1}
            reset          {do_reset}
            invert         {variable ColorScheme; set ColorScheme [expr ($ColorScheme+1)&1]; do_ColorScheme}
           "skip channel 0"    {do_skip "true"}
           "unskip channel 0"  {do_skip "false"}
           "enable QuikLoad"   {set QuikLoad 1}
           "disable QuikLoad"  {set QuikLoad 0}
            default         {InsertLog "Unknown view function $f"}
       }
   }
   
   proc do_arrange {args} {
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable currentgallery
       variable currentoverlap

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           "remove all spectra"   {
                 for {set g 1} {$g <= $numgallery} {incr g} {
                    for {set o 1} {$o <= $numoverlap} {incr o} {
                       if {$dbInUse($g,$o) == 1} {set dbInUse($g,$o) 0}
                    }
                    for {set j 1} {$j <= $MaxTags} {incr j} {
                       if {$dbtagx($g,$j) == 0} {continue}
                       set dbtagx($g,$j) 0; set dbtagy($g,$) 0
                    }
                 }
                 set currentgallery 1
                 set currentoverlap 1
           }
           "remove active spectrum"   {
                 set g $currentgallery
                 for {set o 1} {$o <= $numoverlap} {incr o} {
                     if {$dbInUse($g,$o) == 1} {set dbInUse($g,$o) 0}
                 }
                 for {set j 1} {$j <= $MaxTags} {incr j} {
                     if {$dbtagx($g,$j) == 0} {continue}
                     set dbtagx($g,$j) 0; set dbtagy($g,$j) 0
                 }
                 for {set g 1} {$g <= $numgallery} {incr g} {
                     if {$dbInUse($g,1) == 1} {set currentgallery $g; set currentoverlap 1; break}
                 }
           }

           default                {InsertLog "Unknown arrange function $f"}
       }
   }
   
   proc do_analysis {args} {
       variable QuikLoad
       variable MemSas

       if {$MemSas == 1 && $QuikLoad == 1} {
           InsertLog "analysis functions are not available for MemSas histograms when QuikLoad is enabled"
           return
       }

       set f [lreplace [first $args] 0 0]
   
       switch $f {
           integrate            {do_integrate}
          "peak to total"      {do_peaktototal}
          "max channel counts"  {do_maxcounts}
          "max channel counts (range)" {do_rangemaxcounts}
          "total counts"        {do_totalcounts}
          "total counts (range)"       {do_rangetotalcounts}
          "peak fit"            {do_peakfit}
          "peak fit options"    {}
          "peak find"           {do_peakfind}
          "peak find options"   {}
          "buf fit"            {do_buffit}
          "buf fit options"    {}
           default              {InsertLog "Unknown analysis function $f"}
       }
   }
   
   proc do_tagsfits {args} {

       set f [lreplace [first $args] 0 0]
   
       switch $f {
          "clear tags"    {do_cleartags}
          "clear fits"    {do_clearfits}
           default        {InsertLog "Unknown tagsfits function $f"}
       }
   }
   
   proc do_actions1 {args} {
       variable showit
       variable MemSas
       variable DiscSas
       variable DirName
       variable currentgallery
       variable currentoverlap
       
       upvar #1 dbSpecDim($currentgallery,$currentoverlap) SpecDim
       
       set f [lreplace [first $args] 0 0]
   
       switch $f {
           details   {do_details 0}
           title     {do_title 0}
           create    {
               if {$SpecDim == 1} {do_create1D}
               if {$SpecDim == 2} {do_create2D}
               set showit 0
           }
           delete     {do_delete; set showit 0}
           zero       {do_zero; set showit 0}
           inquire    {do_inquiry [SpectrumName]}
           inquireall {
              if {$MemSas == 1}  {do_inquiry *}
              if {$DiscSas == 1} {do_inquiry [file join $DirName *]}
           }
           reset   {do_reset}
           refresh {}
           default {InsertLog "Unknown analysis function $f"}
       }
   }
   
    proc do_actions2 {args} {
    
       set f [second [first $args]]
       set vl [third [first $args]]

       switch $f {
           tag     {do_pushtag $vl}
           default {InsertLog "Unknown analysis function $args ($f)  ($vl)"}       
       }
    }

    proc do_XMax {vl} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable mode2

       set gBAK $currentgallery
       
       for {set g 1} {$g<=$numgallery} {incr g} {
             if {$dbInUse($g,1) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $gBAK} {continue}
                }

                set currentgallery $g

	upvar #1 dbXMax($currentgallery) XMax

                set XMax $vl
             }
       }

       set currentgallery $gBAK
    }

    proc do_XMin {vl} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable mode2

       set gBAK $currentgallery
       
       for {set g 1} {$g<=$numgallery} {incr g} {
             if {$dbInUse($g,1) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $gBAK} {continue}
                }

                set currentgallery $g

	upvar #1 dbXMin($currentgallery) XMin

                set XMin $vl
             }
       }

       set currentgallery $gBAK
    }

        proc do_YMin {vl} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable mode2

       set gBAK $currentgallery
       
       for {set g 1} {$g<=$numgallery} {incr g} {
             if {$dbInUse($g,1) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $gBAK} {continue}
                }

                set currentgallery $g

	upvar #1 dbYMin($currentgallery) YMin

                set YMin $vl
             }
       }

       set currentgallery $gBAK
    }

    proc do_YMax {vl} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable mode2

       set gBAK $currentgallery
       
       for {set g 1} {$g<=$numgallery} {incr g} {
             if {$dbInUse($g,1) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $gBAK} {continue}
                }

                set currentgallery $g

	upvar #1 dbYMax($currentgallery) YMax

                set YMax $vl
             }
       }

       set currentgallery $gBAK
    }

    proc do_peakfit {} {
       variable backpeakoptions
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
       variable dbfit
       variable MaxFits
       variable nfits
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
      if {$ntags < 6} {InsertLog "6 limits are required"; return}
      
      set tags {}
      for {set i 1} {$i <= 6} {incr i} {
          lappend tags $dbtagx($currentgallery,[expr $ntags-6+$i])
      }
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set use_shapesfile ""
                if {$backpeakoptions(shapesfile) == "yes"} {set use_shapesfile "-shapesfile $shapesfile"}
                
                set execute "exec backpeak  \
                  -spectrum $dbSpecPathName($g,$o) \
                  -bgtype $backpeakoptions(backtype) \
                  -peaktype $backpeakoptions(peaktype) \
                  -weighting $backpeakoptions(weight) \
                  -shape $backpeakoptions(shape) \
                  -order $backpeakoptions(order) \
                  -peaks $backpeakoptions(peaks) \
                  -centroid compute \
                  $use_shapesfile \
                  -limit $tags"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "[last $m]\n"
                } else {
                   InsertLog "Peakfit returns an error: $dbSpecPathName($g,$o): $m"
                   InsertLog "\nUsage:  6 tags are required  \
                      \nfirst 2 define the peak to be fitted\
                      \nremaining 4 tags define 2 background areas which must be on different sides of the peak"
                   return
                }
             }
          }
       }
      
      if {$nfits == $MaxFits} {
          InsertLog "All fits in use"
      } else {
      
         incr nfits
      
         set fchan  [first [first [second $m]]]
         set fnum  [second [first [second $m]]]
         set fvals [second [second $m]]
      
         set dbfit($currentgallery,$nfits) ""
         for {set i 0} {$i < $fnum} {incr i} {
           append dbfit($currentgallery,$nfits) "\[$fchan,[lindex $fvals $i]\],"
           incr fchan
         }
      }
   
    }
    
    proc do_buffit {} {
       variable brumfitoptions
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
      if {$ntags < 3} {InsertLog "at least 3 tags are required"; return}
      
      set tags {}
      for {set i 1} {$i <= $ntags} {incr i} {
          lappend tags $dbtagx($currentgallery,$i)
      }
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec brumfit  \
                  -spectrum $dbSpecPathName($g,$o) \
                  -shape $brumfitoptions(shape) \
                  -widthopt $brumfitoptions(widthopt) \
                  -peakopt $brumfitoptions(peakopt) \
                  -order $brumfitoptions(order) \
                  -limit [first $tags] [second $tags] \
                  -peaks [tail [tail $tags]] \
                  -widths $brumfitoptions(widths) \
                  -skews $brumfitoptions(skews)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "$dbSpecPathName($g,$o)[first $m]\n"
                } else {
                   InsertLog "Brumfit returns an error: $dbSpecPathName($g,$o): $m"
                   InsertLog "\nUsage:  2 + n tags are required  \
                      \nfirst 2 define the range to be considered\
                      \nremaining tags define the peaks to be fitted"
                }
             }
          }
       }
    }
    
    proc  do_peakfind {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       variable peakfindoptions
    
       upvar #1 dbXMin($currentgallery) XMin
       upvar #1 dbXMax($currentgallery) XMax
       
       set peakfindoptions(low-chan) $XMin
       set peakfindoptions(high-chan) $XMax
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
                
                set execute "exec peakfind \
                   -spectrum $dbSpecPathName($g,$o) \
                   -limit $peakfindoptions(low-chan) $peakfindoptions(high-chan) \
                   -fwhm $peakfindoptions(fwhm) \
                   -accept $peakfindoptions(accept)"

                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "$m\n"
                } else {
                   InsertLog "PeakFind returns an error: $m"
                }
             }
          }
       }
    }
    
    proc do_integrate {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
       if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec integrate -limit  $dbtagx($currentgallery,$ntags) $dbtagx($currentgallery,[expr $ntags-1]) -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "$m\n"
                } else {
                   InsertLog "Integrate returns an error: $m"
                }
             }
          }
       }
    }
    
    proc do_peaktototal {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
       if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec peak-to-total -spectrum $dbSpecPathName($g,$o) -limit  $dbtagx($currentgallery,$ntags) $dbtagx($currentgallery,[expr $ntags-1])"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "$m\n"
                } else {
                   InsertLog "PeakToTotal returns an error: $m"
                }
             }
          }
       }
    }
   
    proc do_totalcounts {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable dbSpecPathName
       variable mode2
       
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec total-counts -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "Total-Counts for spectrum $m"
                } else {
                   InsertLog "Total-Counts for spectrum $dbSpecPathName($g,$o)"
                   InsertLog "Total-Counts returns an error: $m"
                }
             }
          }
       }
    }
    
    proc do_rangetotalcounts {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable MaxTags
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec total-counts  $dbtagx($currentgallery,$ntags) $dbtagx($currentgallery,[expr $ntags-1]) -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "Total-Counts from  $dbtagx($currentgallery,$ntags) to $dbtagx($currentgallery,[expr $ntags-1]) for spectrum $m"
                } else {
                   InsertLog "Total-Countsfrom  $dbtagx($currentgallery,$ntags) to $dbtagx($currentgallery,[expr $ntags-1]) for spectrum $dbSpecPathName($g,$o)"
                   InsertLog "Total-Counts returns an error: $m"
                }
             }
          }
       }
    }
   
    proc do_maxcounts {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable dbSpecPathName
       variable mode2
       
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec max-counts -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "Max-Counts for spectrum $m"
                } else {
                   InsertLog "Max-Counts for spectrum $dbSpecPathName($g,$o)"
                   InsertLog "Max-Counts returns an error: $m"
                }
             }
          }
       }
    }
    
    proc do_rangemaxcounts {} {
       variable MaxTags
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable dbtagx
       variable dbtagy
       variable dbSpecPathName
       variable mode2

      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
      
      if {$ntags < 2} {InsertLog "2 limits are required"; return}
      
       for {set g 1} {$g<=$numgallery} {incr g} {
          for {set o 1} {$o<=$numoverlap} {incr o} {
             if {$dbInUse($g,$o) == 1} {
                if {$mode2 == 0} {     ;#  mode is current 
                    if {$g != $currentgallery || $o != $currentoverlap} {continue}
                }
           
                set execute "exec max-counts  $dbtagx($currentgallery,$ntags) $dbtagx($currentgallery,[expr $ntags-1]) -spectrum $dbSpecPathName($g,$o)"
                set rc [catch {eval $execute} m]
      
                if {$rc == 0} {
                   InsertLog "Max-Counts from  $dbtagx($currentgallery,$ntags) to $dbtagx($currentgallery,[expr $ntags-1]) for spectrum $m"
                } else {
                   InsertLog "Max-Countsfrom  $dbtagx($currentgallery,$ntags) to $dbtagx($currentgallery,[expr $ntags-1]) for spectrum $dbSpecPathName($g,$o)"
                   InsertLog "Max-Counts returns an error: $m"
                }
             }
          }
       }
    }
    
   proc do_cont {def} {
       variable DefCont
       if {$def == "true" || $def == 1} {
         set DefCont 1
       } else {
         set DefCont 0
       }
   }

   proc do_skip {skip} {
       variable currentgallery
       variable currentoverlap
       
       upvar #1 dbXMin($currentgallery) XMin
       upvar #1 dbXMax($currentgallery) XMax
   
       upvar #1 dbSpecBase($currentgallery,$currentoverlap) SpecBase
       
       variable Skip0
       
       if {$skip == "true" || $skip == 1} {
         set Skip0 1
       } else {
         set Skip0 0
         if {$XMin == 1} {set XMin $SpecBase}
       }
   }   
    
    proc do_zoom {w} {
       variable MaxTags
       variable dbtagx
       variable currentgallery
       
       upvar #1 dbXMin($currentgallery) XMin
       upvar #1 dbXMax($currentgallery) XMax
       variable dbtagx
       
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
    
       if {$ntags < 1} {InsertLog "You need 1 tag for zoom"; return}
    
        set x $dbtagx($currentgallery,$ntags)
        set range [expr $XMax - $XMin]
        
        if {$w == "In"} {
           set XMin [expr $x - ($range/4)]
           set XMax [expr $x + ($range/4)]
        } else {
           set XMin [expr $x - $range]
           set XMax [expr $x + $range]        
        }
    }
   
   proc do_pushtag {x} {
       variable currentgallery
       variable numgallery
       variable MaxTags
       
       variable dbInUse
       variable mode2
       variable dbtagx
       variable dbtagy
       
       for {set g 1} {$g <= $numgallery} {incr g} {
           if {$dbInUse($g,1) == 1} {
               if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
               
               upvar #1 dbSpecRawData($g,1) SpecRawData   ;# take counts from overlap 1
               
               set done 0
               for {set j 1} {$j <= $MaxTags} {incr j} {
                   if {$dbtagx($g,$j) != 0} {continue}
                   
                   set dbtagx($g,$j) [expr round($x)]
                   set dbtagy($g,$j) [lindex $SpecRawData $dbtagx($g,$j)]
                   
                   incr done
                   break    ;#  done 
               }
               if {$done == 0} {InsertLog "too many tags for gallery $g. max supported = $MaxTags"}
           }
       }
   }
   
   proc do_reset {} {
       variable currentgallery
       variable numgallery
       variable dbInUse
       variable mode2
       
       variable CustomYView
       variable WindowSize
       variable SpecLimit
       
       for {set g 1} {$g<=$numgallery} {incr g} {
         if {$dbInUse($g,1) == 1} {
           if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
           
       upvar #1 dbXMin($g) XMin
       upvar #1 dbXMax($g) XMax
       upvar #1 dbYMin($g) YMin
       upvar #1 dbYMax($g) YMax
       
       upvar #1 dbSpecBase($g,1) SpecBase
       upvar #1 dbSpecRange($g,1) SpecRange
           
           set XMin $SpecBase
           set XMax $SpecRange

           set WindowSize [expr $XMax - $XMin]
           
           set YMin  0
           set YMax  0
       
         }
       }
       
       set CustomYView 0
       set SpecLimit 0
   }
   
   proc do_expand {} {
       variable currentgallery
       variable currentoverlap
       variable numgallery
       variable numoverlap
       variable dbInUse
       variable mode2
       variable MaxTags
       variable dbtagx
       variable dbXMin
       variable dbXMax

       variable WindowSize
   
      set ntags 0
      for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
      }
       
       if {$ntags < 2} {InsertLog "You need 2 tags for expand"; return}
   
       for {set g 1} {$g<=$numgallery} {incr g} {
          if {$dbInUse($g,1) == 1} {
            if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
           
       if {$dbtagx($currentgallery,[expr $ntags-1]) < $dbtagx($currentgallery,$ntags)} {
           set dbXMin($g) $dbtagx($currentgallery,[expr $ntags-1])
           set dbXMax($g) $dbtagx($currentgallery,$ntags)
       } else {
           set dbXMin($g) $dbtagx($currentgallery,$ntags)
           set dbXMax($g) $dbtagx($currentgallery,[expr $ntags-1])
       }
       
           set WindowSize [expr $dbXMax($currentgallery) - $dbXMin($currentgallery)] 
       
          }
        }
       
        do_poptag
        do_poptag
   }

   
   proc do_cleartags {} {
       variable currentgallery
       variable numgallery
       variable dbInUse
       variable MaxTags
       variable mode2
       variable dbtagx
   
       for {set g 1} {$g<=$numgallery} {incr g} {
         if {$dbInUse($g,1) == 1} {
            if {$mode2 == 0 && $g != $currentgallery} {continue}  ;# mode is current
           
            for {set j 1} {$j <= $MaxTags} {incr j} {
                if {$dbtagx($g,$j) == 0} {continue}
                set dbtagx($g,$j) 0; set dbtagy($g,$j) 0
            }
         }
       }
   }
   
   proc do_clearfits {} {
       variable nfits
       variable MaxFits
       variable currentgallery
       variable dbtagx
   
       set nfits 0
       for {set i 1} {$i<=$MaxFits} {incr i} {set dbfit($currentgallery,$i) ""}
   
   }
   
   proc do_poptag {} {
       variable MaxTags
       variable currentgallery
       variable dbtagx
        
       set ntags 0
       for {set j 1} {$j <= $MaxTags} {incr j} {
          if {$dbtagx($currentgallery,$j) == 0} {continue}
          incr ntags
       }
   
       if {$ntags < 1} {return}
       
       set dbtagx($currentgallery,$ntags) 0
       set dbtagy($currentgallery,$ntags) 0
   }

